searchNodes=[{"doc":"Check SPF for a specific sender and possible options. The Spf.check/2 function takes a sender and possible options and returns an evaluation context that contains the verdict and some statistics of the evaluation. Example iex&gt; unless File . dir? ( &quot;tmp&quot; ) , do : File . mkdir ( &quot;tmp&quot; ) iex&gt; File . write ( &quot;tmp/zone.txt&quot; , &quot;&quot;&quot; ...&gt; example.com TXT v=spf1 -all exp=why.%{d} ...&gt; why.example.com TXT %{d}: %{i} is not one of our MTA&#39;s ...&gt; &quot;&quot;&quot; ) :ok iex&gt; ctx = Spf . check ( &quot;example.com&quot; , dns : &quot;tmp/zone.txt&quot; ) iex&gt; { ctx . verdict , ctx . reason , ctx . explanation } { :fail , &quot;spf[0] -all&quot; , &quot;example.com: 127.0.0.1 is not one of our MTA&#39;s&quot; }","ref":"Spf.html","title":"Spf","type":"module"},{"doc":"Check SPF for given sender and possible options. Options include: dns: filepath or zonedata to pre-populate the context's DNS cache helo: the helo presented by sending MTA, defaults to sender ip: ipv4 or ipv6 address, in binary, of sending MTA, defaults to 127.0.0.1 log: a user log/4 function to relay notifications, defaults to nil verbosity how verbose the notifications should be (0..5), defaults to 3 Examples iex&gt; zone = &quot;&quot;&quot; ...&gt; example.com TXT v=spf1 +all ...&gt; &quot;&quot;&quot; iex&gt; Spf . check ( &quot;example.com&quot; , dns : zone ) |&gt; Map . get ( :verdict ) :pass","ref":"Spf.html#check/2","title":"Spf.check/2","type":"function"},{"doc":"Functions to create, access and update an SPF evaluation context. Many functions take and return an evaluation context whose purpose is to store information gathered during the evaluation. This includes a dns cache, an ip lookup table that maps prefixes to SPF terms that named them, a stack for recursive evaluations, as well as some statistics around DNS mechanisms seen and void DNS responses seen.","ref":"Spf.Context.html","title":"Spf.Context","type":"module"},{"doc":"Updates context.ipt with one or more { prefix/0 , iptval/0 }-pairs. When given a list op ip's, they all will be be updated with given iptval/0 which records the SPF record and term (including the qualifier) that attributed the ip or ip's. The dual parameter contains the dual-cidr lengths to apply to the given ip addresses.","ref":"Spf.Context.html#addip/4","title":"Spf.Context.addip/4","type":"function"},{"doc":"Updates context with given error , reason and verdict . When verdict is nil, context.verdict is not updated. This allows for setting error conditions whose impact is to be evaluated at a later stage.","ref":"Spf.Context.html#error/5","title":"Spf.Context.error/5","type":"function"},{"doc":"Returns a previous SPF string given either its domain of nth -tracking number. Used for reporting rather than evalutation an SPF record.","ref":"Spf.Context.html#get_spf/2","title":"Spf.Context.get_spf/2","type":"function"},{"doc":"Updates context 's message queue and, if available, calls the user supplied log function. The log/4 is called with: context the current context/state of the evalution facility an atom denoting which part of the program emitted the event severity an atom describing the severity msg a binary with event details","ref":"Spf.Context.html#log/4","title":"Spf.Context.log/4","type":"function"},{"doc":"Returns true if new_domain constitues a loop for given context , false otherwise. Loops may occur when two SPF records (eventually) include or redirect to each other and is considered a permanent error.","ref":"Spf.Context.html#loop?/2","title":"Spf.Context.loop?/2","type":"function"},{"doc":"Returns a new Spf.Context.t/0 for given sender . Options include: dns: , filepath or binary with zonedata (defaults to nil) helo: , sender's helo string to use (defaults to sender ) ip: , sender ip to use (defaults to 127.0.0.1 ) log: , user supplied log function (defaults to nil) verbosity: , log level 0..5 to use (defaults to 4 ) The initial domain is derived from given sender . The default for ip is likely to traverse all SPF mechanisms during evaluation, gathering as much information as possible. Set ip: to a real IPv4 or IPv6 address to check a policy for that specific address. The context is used for the entire SPF evaluation, including during any recursive calls. When evaluating an include mechanism, the current state (a few selected context properties) is pushed onto an internal stack and a new domain is set. After evaluating the include mechanism, the state if popped and the results are processed according to the include -mechanism's qualifier. When evaluating a redirect modifier, the current state is altered for the new domain specified by the modifier.","ref":"Spf.Context.html#new/2","title":"Spf.Context.new/2","type":"function"},{"doc":"Pop the previous state of given context from its stack. Before evaluating an include mechanism, the current SPF's record state is pushed onto the stack. This function restores that state from the stack.","ref":"Spf.Context.html#pop/1","title":"Spf.Context.pop/1","type":"function"},{"doc":"Push the current state of given context onto its stack and re-init the context. The details of the current SPF record are pushed onto a stack and the context is re-initialized for retrieving, parsing and evaluate a new include d record.","ref":"Spf.Context.html#push/2","title":"Spf.Context.push/2","type":"function"},{"doc":"Reinitializes current context for given domain of a redirect modifier. When a redirect modifier is encountered it basically replaces the current SPF record and the context is modified accordingly..","ref":"Spf.Context.html#redirect/2","title":"Spf.Context.redirect/2","type":"function"},{"doc":"Given a current context and a range , return the SPF term in that range. Retrieves a slice of the context.spf current record being evaluated. Used for logging events.","ref":"Spf.Context.html#spf_term/2","title":"Spf.Context.spf_term/2","type":"function"},{"doc":"Split an email address into a local and a domain part. The local part is left to the left-most @ , if there is no local part it defaults to &quot;postmaster&quot;. Note that splitting an empty string yields {&quot;postmaster&quot;, &quot;&quot;} .","ref":"Spf.Context.html#split/1","title":"Spf.Context.split/1","type":"function"},{"doc":"If test is true, logs the given msg with its facility and severity . A convencience function to quickly perform some test (in the call) and, if true, log it as well.","ref":"Spf.Context.html#test/5","title":"Spf.Context.test/5","type":"function"},{"doc":"Adds delta to counter and returns updated context . Valid counters include: :num_spf , the number of SPF records seen :num_dnsm the number of DNS mechanisms seen :num_dnsq the number of DNS queries performed :num_dnsv the number of void DNS queries seen :num_checks the number of checks performed :num_warn the number of warnings seen :num_error the number of errors see (may not be fatal) :depth the current recursion depth","ref":"Spf.Context.html#tick/3","title":"Spf.Context.tick/3","type":"function"},{"doc":"A {qualifier, nth, term} tuple, where nth is the nth SPF record where term was found. The context's ip lookup table stores these tuples thus tracking which term in which SPF record provided a qualifier for a prefix. Since an evaluation may involve multiple SPF records, each prefix actually stores a list of these tuples. Once the sender's ip has a longest prefix match, the qualifier will tell how the mechanism at hand matches.","ref":"Spf.Context.html#t:iptval/0","title":"Spf.Context.iptval/0","type":"type"},{"doc":"A Pfx.prefix/0 .","ref":"Spf.Context.html#t:prefix/0","title":"Spf.Context.prefix/0","type":"type"},{"doc":"An SPF evaluation context.","ref":"Spf.Context.html#t:t/0","title":"Spf.Context.t/0","type":"type"},{"doc":"A Spf.Tokens.token/0 .","ref":"Spf.Context.html#t:token/0","title":"Spf.Context.token/0","type":"type"},{"doc":"An SPF evaluation result.","ref":"Spf.Context.html#t:verdict/0","title":"Spf.Context.verdict/0","type":"type"},{"doc":"A simple DNS caching resolver for SPF evaluations. During an SPF evaluation all DNS responses are cached. Since the cache lasts only for the duration of the evaluation, TTL values are ignored. The cache allows for reporting on DNS data acquired during the evaluation. By preloading the cache, using Spf.DNS.load/2 , new records can be tested. The caching resolver also tracks the number of DNS queries made and the number of void queries seen. Example iex&gt; zonedata = &quot;&quot;&quot; ...&gt; example.com TXT v=spf1 +all ...&gt; &quot;&quot;&quot; iex&gt; ctx = Spf.Context . new ( &quot;example.com&quot; , dns : zonedata ) iex&gt; { _ctx , result } = Spf.DNS . resolve ( ctx , &quot;example.com&quot; , type : :txt ) iex&gt; result { :ok , [ &quot;v=spf1 +all&quot; ] }","ref":"Spf.DNS.html","title":"Spf.DNS","type":"module"},{"doc":"Finds a domain name 's start of authority and contact. SPF evaluation might require evaluating multiple records of different domains. This function allows for reporting the owner and contact for each SPF record encountered. CNAME's are ignored since the goal is to find the authoritative zone for a given (sub)domain name . Returns {:ok, domain, authority, contact} , or {:error, :err_code} The given name does not need to actually exist, the aim is to find the owner of the domain the name belongs to. Examples iex&gt; Spf.Context . new ( &quot;example.com&quot; ) ...&gt; |&gt; Spf.DNS . authority ( &quot;non-existing.example.com&quot; ) { :ok , &quot;non-existing.example.com&quot; , &quot;example.com&quot; , &quot;noc@dns.icann.org&quot; }","ref":"Spf.DNS.html#authority/2","title":"Spf.DNS.authority/2","type":"function"},{"doc":"Checks validity of a domain name and returns {:ok, name} or {:error, reason} Checks that the domain name: is an ascii string is less than 254 chars long has labels that are 1..63 chars long, and has at least 2 labels has a valid ldh-toplabel Examples iex&gt; check_domain ( &quot;com&quot; ) { :error , &quot;not multi-label&quot; } iex&gt; check_domain ( &quot;.example.com&quot; ) { :error , &quot;empty label&quot; } iex&gt; check_domain ( &quot;example..com&quot; ) { :error , &quot;empty label&quot; } iex&gt; check_domain ( &lt;&lt; 128 &gt;&gt; &lt;&gt; &quot;.com&quot; ) { :error , &quot;contains non-ascii characters&quot; } iex&gt; check_domain ( &quot;example.-com&quot; ) { :error , &quot;tld starts with hyphen&quot; } iex&gt; check_domain ( &quot;example.com-&quot; ) { :error , &quot;tld ends with hyphen&quot; } iex&gt; check_domain ( &quot;example.c%m&quot; ) { :error , &quot;tld not ldh&quot; } iex&gt; check_domain ( &quot;example.c0m.&quot; ) { :ok , &quot;example.c0m&quot; }","ref":"Spf.DNS.html#check_domain/1","title":"Spf.DNS.check_domain/1","type":"function"},{"doc":"Filters the dns_result/0 , keeps only the rrdata's for which fun returns a truthy value. If the dns_result is actually an error, it is returned untouched. Examples iex&gt; zonedata = &quot;&quot;&quot; ...&gt; example.com TXT v=spf1 -all ...&gt; example.com TXT another txt record ...&gt; &quot;&quot;&quot; iex&gt; ctx = Spf.Context . new ( &quot;example.com&quot; , dns : zonedata ) iex&gt; { _ctx , dns_result } = resolve ( ctx , &quot;example.com&quot; , type : :txt ) iex&gt; iex&gt; dns_result { :ok , [ &quot;another txt record&quot; , &quot;v=spf1 -all&quot; ] } iex&gt; iex&gt; filter ( dns_result , &amp; Spf.Eval . spf? / 1 ) { :ok , [ &quot;v=spf1 -all&quot; ] } iex&gt; Spf.DNS . filter ( { :error , :nxdomain } , &amp; Spf.Eval . spf? / 1 ) { :error , :nxdomain }","ref":"Spf.DNS.html#filter/2","title":"Spf.DNS.filter/2","type":"function"},{"doc":"Returns a cached dns_result/0 for given name , type and context or a cache_miss/0 . The result returned is one of: {:error, :cache_miss} , for a cache miss {:error, reason} , for a cache hit (of a previous negative result) {:ok, rrs} , for a cache hit (where rrs is a list of rrdata's). Where reason includes: :nxdomain :servfail :timeout :zero_answers Note that this function normalizes given name and unrolls CNAME(s) and does not make any actual DNS requests nor does it do any statistics. Example iex&gt; zonedata = &quot;&quot;&quot; ...&gt; example.net CNAME example.com ...&gt; EXAMPLE.COM A 1.2.3.4 ...&gt; &quot;&quot;&quot; iex&gt; Spf.Context . new ( &quot;some.domain.tld&quot; , dns : zonedata ) ...&gt; |&gt; Spf.DNS . from_cache ( &quot;example.net&quot; , :a ) { :ok , [ &quot;1.2.3.4&quot; ] }","ref":"Spf.DNS.html#from_cache/3","title":"Spf.DNS.from_cache/3","type":"function"},{"doc":"Populates the dns cache of given context , with dns 's zonedata. dns can be a path to an existing file, a multi-line binary containing individual RR-records per line or a list thereof. The cache is held in the context under the :dns key and is a simple map: {name, rrtype} -&gt; [rdata] . Lines should be formatted as name rr-type data , or name error where rr-type is a , aaaa , cname , mx , ns , ptr , soa , spf , or txt error is formerr , nxdomain , servfail , timeout or zero_answers The second format will set the rdata for given name to given error for all known rr-type 's. Unknown rr-types are ignored and logged as a warning during preloading. Example iex&gt; zonedata = &quot;&quot;&quot; ...&gt; example.com TXT v=spf1 +all ...&gt; example.com A timeout ...&gt; EXAMPLE.NET servfail ...&gt; &quot;&quot;&quot; iex&gt; ctx = Spf.Context . new ( &quot;some.domain.tld&quot; ) ...&gt; |&gt; Spf.DNS . load ( zonedata ) iex&gt; iex&gt; { _ctx , result } = Spf.DNS . resolve ( ctx , &quot;example.com&quot; , type : :txt ) iex&gt; result { :ok , [ &quot;v=spf1 +all&quot; ] } iex&gt; iex&gt; Spf.DNS . resolve ( ctx , &quot;example.com&quot; , type : :a ) |&gt; elem ( 1 ) { :error , :timeout } iex&gt; iex&gt; Spf.DNS . resolve ( ctx , &quot;example.net&quot; , type : :a ) |&gt; elem ( 1 ) { :error , :servfail } iex&gt; iex&gt; ctx . dns %{ { &quot;example.com&quot; , :a } =&gt; [ error : :timeout ] , { &quot;example.com&quot; , :txt } =&gt; [ &quot;v=spf1 +all&quot; ] , { &quot;example.net&quot; , :a } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :aaaa } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :cname } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :mx } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :ns } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :ptr } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :soa } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :spf } =&gt; [ error : :servfail ] , { &quot;example.net&quot; , :txt } =&gt; [ error : :servfail ] }","ref":"Spf.DNS.html#load/2","title":"Spf.DNS.load/2","type":"function"},{"doc":"Normalize a domain name by trimming, downcasing and removing any trailing dot. The validity of the domain name is not checked. Examples iex&gt; normalize ( &quot;Example.COM.&quot; ) &quot;example.com&quot; iex&gt; normalize ( &quot;EXAMPLE.C%M&quot; ) &quot;example.c%m&quot;","ref":"Spf.DNS.html#normalize/1","title":"Spf.DNS.normalize/1","type":"function"},{"doc":"Resolves a query, updates the cache and returns a { ctx , dns_result/0 }-tuple. Returns: {ctx, {:error, reason}} if a DNS error occurred, or {ctx, {:ok, [rrs]}} where rrs is a list of rrdata's Although a result with ZERO answers is technically not a DNS error, it will be reported as an error. Error reasons include: :zero_answers :illegal_name :timeout :nxdomain :servfail other Options include: type: , which defaults to :a stats , which defaults to true When stats is false , void DNS responses ( :nxdomain or :zero_answers ) are not counted.","ref":"Spf.DNS.html#resolve/3","title":"Spf.DNS.resolve/3","type":"function"},{"doc":"Return all acquired DNS RR's in a flat list of printable lines. Note that RR's with multiple entries in their rrdata are listed individually, so the output can be copy/paste'd into a local dns.txt pre-cache to facilitate experimentation with RR records. The lines are sorted such that domains and subdomains are kept together as much as possible. Example iex&gt; zonedata = &quot;&quot;&quot; ...&gt; example.com TXT v=spf1 -all ...&gt; a.example.com A 1.2.3.4 ...&gt; b.example.com AaAa timeout ...&gt; &quot;&quot;&quot; iex&gt; ctx = Spf.Context . new ( &quot;example.com&quot; , dns : zonedata ) iex&gt; Spf.DNS . to_list ( ctx , valid : :true ) ...&gt; |&gt; Enum . map ( fn x -&gt; String . replace ( x , ~r/ \\s +/ , &quot; &quot; ) end ) [ &quot;example.com TXT \\&quot; v=spf1 -all \\&quot; &quot; , &quot;a.example.com A 1.2.3.4&quot; ] iex&gt; Spf.DNS . to_list ( ctx , valid : false ) ...&gt; |&gt; Enum . map ( fn x -&gt; String . replace ( x , ~r/ \\s +/ , &quot; &quot; ) end ) [ &quot;b.example.com AAAA TIMEOUT&quot; ] iex&gt; iex&gt; Spf.DNS . to_list ( ctx ) ...&gt; |&gt; Enum . map ( fn x -&gt; String . replace ( x , ~r/ \\s +/ , &quot; &quot; ) end ) [ &quot;example.com TXT \\&quot; v=spf1 -all \\&quot; &quot; , &quot;a.example.com A 1.2.3.4&quot; , &quot;b.example.com AAAA TIMEOUT&quot; ]","ref":"Spf.DNS.html#to_list/2","title":"Spf.DNS.to_list/2","type":"function"},{"doc":"An {:error, :cache_miss} -tuple","ref":"Spf.DNS.html#t:cache_miss/0","title":"Spf.DNS.cache_miss/0","type":"type"},{"doc":"A DNS result in the form of an ok/error-tuple.","ref":"Spf.DNS.html#t:dns_result/0","title":"Spf.DNS.dns_result/0","type":"type"},{"doc":"Functions to evaluate an SPF context.","ref":"Spf.Eval.html","title":"Spf.Eval","type":"module"},{"doc":"Given a context retrieve and evaluate the associated SPF record. After an (attempted) evaluation, returns an updated context where: :verdict will be one of :pass, :fail, :softfail, :neutral :reason show the mechanism responsible for the verdict :explanation the expanded explain-string (if possible and applicable) :error in case one occurred :ipt which maps the prefixes seen during evaluation to their source :msg which lists log messages accumulated during evaluation and other fields containing information gathered during the evaluation. The context is passed around accumulating information and tracks the state of the evaluation. Its :log is either nil or points to a log_function/4 that then called with the context , facility , severity and a message so it can dump it to screen or somewhere else.","ref":"Spf.Eval.html#evaluate/1","title":"Spf.Eval.evaluate/1","type":"function"},{"doc":"Say whether str contains the start of an SPF string. Leading whitespace is not considered an error, although technically it is a syntax error.","ref":"Spf.Eval.html#spf?/1","title":"Spf.Eval.spf?/1","type":"function"},{"doc":"Returns true if name is a validated name for given domain . The dns_result should contain the ip addresses associated with given name . If any of the ip adresss match the given ip , the name is a validated domain name for given domain . If the exact flag is true, then the name is also required to end with given domain as well. Note that when trying to validate names during the expansion of the p-marco, this flag will be false.","ref":"Spf.Eval.html#validate?/5","title":"Spf.Eval.validate?/5","type":"function"},{"doc":"","ref":"Spf.Eval.html#t:context/0","title":"Spf.Eval.context/0","type":"type"},{"doc":"","ref":"Spf.Eval.html#t:dns_result/0","title":"Spf.Eval.dns_result/0","type":"type"},{"doc":"Functions to parse an SPF string or explain string for a given SPF context . The parser also performs expansion during the semantic checks, so both functions take an SPF context as their only argument.","ref":"Spf.Parser.html","title":"Spf.Parser","type":"module"},{"doc":"Parse context 's explain string and store the result under the :explanation key. In case of any syntax errors, sets the explanation string to an empty string.","ref":"Spf.Parser.html#explain/1","title":"Spf.Parser.explain/1","type":"function"},{"doc":"Parse context 's SPF string and store the result under the :ast key. The parser will parse the entire record so as to find as many problems as possible. The parser will log notifications for: ignoring an include'd explain modifier for each DNS mechanism encountered The parser will log warnings for: an SPF string length longer than 512 characters any residue text in the SPF string after parsing when an exp modifier is present, but the SPF record cannot fail SPF records with implicit endings ignoring a redirect modifier because the all mechanism is present each ignored term occurring after an all mechanism the use of the ptr mechanism (which is not recommended) the use of the p-macro (also not recommended) repeated whitespace to separate terms use of tab character(s) to sepearate terms ignoring an unknown modifier a redirect modifer, when no all is present, that is not the last term The parser will log an error for: repeated modifier terms syntax errors in domain specifications syntax errors in dual-cidr notations invalid IP addresses unknown terms that are not unknown modifiers The logging simply adds messages to the context.msg list but, when logging an error, the context.error and context.reason are also set. Since the parser does not stop at the first error, the context.error and context.reason show the details of the last error seen. If given context also has a function reference stored in context.log , it is called with 4 arguments: context facility , an atom denoting which part sent the message severity , an atom like :info, :warn, :error, :debug msg , the message string In the absence of an error, context.ast is fit for evaluation.","ref":"Spf.Parser.html#parse/1","title":"Spf.Parser.parse/1","type":"function"},{"doc":"","ref":"Spf.Parser.html#t:context/0","title":"Spf.Parser.context/0","type":"type"},{"doc":"","ref":"Spf.Parser.html#t:token/0","title":"Spf.Parser.token/0","type":"type"},{"doc":"Combinators to turn an SPF string or an explain string into a list of token/0 's. This module provides two combinators: tokenize_spf/0 , which turns an SPF string into a list of tokens tokenize_exp/0 , which turns an explain string into tokens which are used by Spf.Parser for lexical analysis of an input string. The start_token/6 and token/6 are helper functions for internal use by Spf.Tokens that should've been private, but it seems they cannot be.","ref":"Spf.Tokens.html","title":"Spf.Tokens","type":"module"},{"doc":"Token {:exp_str, [token], range} . Where token 's include: :expand , :toplabel , :literal or :whitespace . After expanding the domain spec, of an :exp -token into a domain name, its TXT RR is retrieved. That RR's text value is called the explain-string. This string is the only place where macro-letters c , r , or t are allowed. The list of tokens can then be expanded into the final explanation by the parser.","ref":"Spf.Tokens.html#tokenize_exp/0","title":"Spf.Tokens.tokenize_exp/0","type":"function"},{"doc":"Combinator that creates a list of tokens for the SPF terms found in an input string.","ref":"Spf.Tokens.html#tokenize_spf/0","title":"Spf.Tokens.tokenize_spf/0","type":"function"},{"doc":"","ref":"Spf.Tokens.html#t:combinator/0","title":"Spf.Tokens.combinator/0","type":"type"},{"doc":"qualifier = ?+ / ?- / ?~ / ??","ref":"Spf.Tokens.html#t:q/0","title":"Spf.Tokens.q/0","type":"type"},{"doc":"range denotes a token's start..stop -slice in the input string.","ref":"Spf.Tokens.html#t:range/0","title":"Spf.Tokens.range/0","type":"type"},{"doc":"A token represented as a tuple: {type, list, range} . Where: type is an atom which denotes the token type/0 list may be empty or contain one or more values (including subtokens) range is the start..stop -slice in the input string","ref":"Spf.Tokens.html#t:token/0","title":"Spf.Tokens.token/0","type":"type"},{"doc":"The token's type . There are several classes of tokens: the version: :version , a mechanism: :a, :all, :exists, :exp, :include, :ip4, :ip6, :mx, :ptr a modifier: :exp, :redirect , an explain string: exp_str , an unknown modifier: unknown_mod , an unknown token: unknown whitespace: :whitespace , a subtoken: :domspec, :dual_cidr, :qualifier , :expand, :literal, :toplabel Subtokens may appear as part of another token's value. :exp_str is produced by tokenizing the explain string. After expanding the domain-spec of modifier exp=domain-spec into a domain name, that domain's TXT RR is retrieved and tokenized for later expansion into an explanation string. This only happens when the SPF verdict is :fail and the exp modifier is present and has a valid domain name. The :whitespace token will match both spaces and tab characters in order to be able to warn about multiple spaces and/or tab characters being used. Use of a tab character is technically a syntax error, but this library only warns about its use. The :unknown token is tried as a last resort and matches any non-space sequence. When matched, it usually means the SPF string has a syntax error.","ref":"Spf.Tokens.html#t:type/0","title":"Spf.Tokens.type/0","type":"type"},{"doc":"spfcheck is a command line tool to examine and debug SPF records. Usage Usage: spfcheck [options] [sender ...] where sender = [localpart@]domain and localpart defaults to 'postmaster' Options: -H, --help print this message and exit -d, --dns=filepath file with DNS RR records to prepopulate the DNS cache -h, --helo=string sending MTA helo/ehlo identity (defaults to nil) -i, --ip=string sending MTA IPv4/IPv6 address (defaults to 127.0.0.1) -r, --report=string either &quot;all&quot; or one of more letters of &quot;vsewpdat&quot; (see below) -v, --verbosity set logging noise level (0..5), default is 4 (informational) -w, --width=NUM limits line length to increase readability (defaults to 60) --no-color turn off colors for log messages --no-markdown turn off markdown formatting for reports The default is to simply print the verdict and some stats to stdout and print notification messages to stderr. spfcheck passes the rfc7208 test suite and should be reasonably rfc7208 compliant. % spfcheck example.com --no-color example.com %spf[0]-spf-note: &gt; spfcheck(example.com, 127.0.0.1, example.com) example.com %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.com - [&quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot;, &quot;v=spf1 -all&quot;] example.com %spf[0]-eval-note: &gt; spf[0] -all - matches example.com %spf[0]-dns-info: &gt; DNS QUERY (2) soa example.com - [{&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600}] domain : example.com ip : 127.0.0.1 sender : example.com verdict : fail reason : spf[0] -all owner : example.com contact : noc@dns.icann.org num_spf : 1 num_dnsm : 0 num_dnsq : 1 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 0 explanation: Batchmode If no sender is given on the command line, spfcheck will read stdin for the domains (and options) to check. In this case, the verdict(s) are output on stdout in csv-format as each domain is (sequentially) evaluated. % cat senders.txt example.com me@example.net -i 1.2.3.4 % cat domains.txt | spfcheck -v 0 domain,ip,sender,verdict,reason,owner,contact,num_spf,num_dnsm,num_dnsq,num_dnsv,num_checks,num_warn,num_error,duration,explanation &quot;example.com&quot;,&quot;127.0.0.1&quot;,&quot;example.com&quot;,:fail,&quot;spf[0] -all&quot;,&quot;example.com&quot;,&quot;noc@dns.icann.org&quot;,1,0,1,0,1,0,0,1,&quot;&quot; &quot;example.net&quot;,&quot;1.2.3.4&quot;,&quot;me@example.net&quot;,:fail,&quot;spf[0] -all&quot;,&quot;example.net&quot;,&quot;noc@dns.icann.org&quot;,1,0,1,0,1,0,0,0,&quot;&quot; DNS flag The -d flag can be used to either point to local file with RR-records or specify DNS data on the command line. If the file exists, it is read and used to prepopulate the cache. Otherwise, the text will be read as DNS data. This makes it possible to try out records before publishing them in DNS. That file should contain 1 RR record per line using the simple domain type rdata format. All domain 's are taken to be relative to root ('.'). % spfcheck example.com -v 0 -d &quot;example.com TXT v=spf1 +all&quot; domain : example.com ip : 127.0.0.1 sender : example.com verdict : pass reason : spf[0] +all num_spf : 1 num_dnsm : 0 num_dnsq : 1 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 1 explanation: # Or using a file % cat tmp/zonedata.txt # comments are ignored as are empty lines example.com TXT v=spf1 -all exp=why.%{d} example.com TXT just another txt record why.example.com TXT %{d}: %{i} is not one of our MTAs % spfcheck example.com -v 0 -d tmp/zonedata.txt domain : example.com ip : 127.0.0.1 sender : example.com verdict : fail reason : spf[0] -all num_spf : 1 num_dnsm : 0 num_dnsq : 2 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 1 explanation: example.com: 127.0.0.1 is not one of our MTAs spfcheck counts the number of dns mechanisms seen (dnsm), the number of queries performed (dnsq) and the number of void dns queries seen (dnsv). If the evaluation took more than 10 dns mechanisms, the verdict is modified accordingly. Helo flag The -h allows for setting the EHLO domain name and defaults to given sender . Note that spfcheck only checks SPF for sender , so this is only useful when checking the expansion of the %{h} -macro in a policy. Ip flag The -i flag is used to set sender's IP to either an IPv4 or an IPv6 address, it defaults to 127.0.0.1 as an unlikely address to be authorized by anyone. The goal is to go down the rabbit hole as far as possible and check the entire nested SPF policy for given sender . No color The --no-color flag disables the use of colors in log messages, which is better when redirecting logging to a file. For example: %cat tmp/domains.txt example.com me@example.net -i 1.2.3.4 % cat tmp/domains.txt | spfcheck -v 5 --no-color 2&gt;tmp/log.txt &gt; tmp/checked.csv % cat tmp/log.txt example.com %spf[0]-ctx-debug: &gt; created context for example.com example.com %spf[0]-spf-note: &gt; spfcheck(example.com, 127.0.0.1, example.com) example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot; example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;v=spf1 -all&quot; example.com %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.com - [&quot;v=spf1 -all&quot;, &quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot;] example.com %spf[0]-eval-info: &gt; spf[0] -all - matches example.com %spf[0]-dns-debug: &gt; added {example.com, soa} -&gt; {&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600} example.com %spf[0]-dns-info: &gt; DNS QUERY (2) soa example.com - [{&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600}] example.net %spf[0]-ctx-debug: &gt; created context for example.net example.net %spf[0]-spf-note: &gt; spfcheck(example.net, 1.2.3.4, me@example.net) example.net %spf[0]-dns-debug: &gt; added {example.net, txt} -&gt; &quot;v=spf1 -all&quot; example.net %spf[0]-dns-debug: &gt; added {example.net, txt} -&gt; &quot;5fpl1ghm7scnth0907z0pft8c79lvc8t&quot; example.net %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.net - [&quot;5fpl1ghm7scnth0907z0pft8c79lvc8t&quot;, &quot;v=spf1 -all&quot;] example.net %spf[0]-eval-info: &gt; spf[0] -all - matches example.net %spf[0]-dns-debug: &gt; added {example.net, soa} -&gt; {&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600} example.net %spf[0]-dns-info: &gt; DNS QUERY (2) soa example.net - [{&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600}] Report flag The -r flag can be used to print out some information, topics include: v the verdict and some statistics s the spf records seen and their authority information e the errors seen w the warnings seen p the prefixes collected d DNS information collected a the AST for the (first) SPF record t the tokens for the (last) SPF record seen In case no -r flag is used, spfcheck will simply print out the verdict. % spfcheck example.com -v 0 -r s --no-markdown [0] example.com -- (example.com, noc@dns.icann.org) v=spf1 -all Here, the SPF section lists all SPF records seen along with the authoritative domain and its DNS admin email. Verbosity flag The -v flag controls the verbosity level of logging on stderr: 0 - no messages at all 1 - errors 2 - warnings 3 - notifications 4 - informational 5 - debug % spfcheck example.com -v 5 --no-color example.com %spf[0]-ctx-debug: &gt; created context for example.com example.com %spf[0]-spf-note: &gt; spfcheck(example.com, 127.0.0.1, example.com) example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;v=spf1 -all&quot; example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot; example.com %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.com - [&quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot;, &quot;v=spf1 -all&quot;] example.com %spf[0]-spf-note: &gt; SPF (0): [&quot;v=spf1 -all&quot;] example.com %spf[0]-eval-info: &gt; spf[0] -all - matches domain : example.com ip : 127.0.0.1 sender : example.com verdict : fail reason : spf[0] -all num_spf : 1 num_dnsm : 0 num_dnsq : 1 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 1 explanation: Width flag Finally, the -w flag can be used to control the width used when printing information of certain topics. Primarily meant so a markdown formatted report can be easily converted to pdf. Default value is 60, but you can make it as wide as necessary.","ref":"Spfcheck.html","title":"Spfcheck","type":"module"},{"doc":"Main entry point for spfcheck cli command.","ref":"Spfcheck.html#main/1","title":"Spfcheck.main/1","type":"function"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"[ ] add flags to customize title &amp; author in markdown's metadata [ ] add flag to just create a dot-file that visualizes the domain's SPF (?)","ref":"changelog.html#unreleased","title":"Changelog - [unreleased]","type":"extras"},{"doc":"changed warning when exceeding 512 chars now shows offending SPF domain name &quot;seen before&quot;-warning changed into &quot;multiple entries&quot;-warning (less confusing) parser errors now correctly logged as :parse-errors instead of :eval-errors added warning about inconsistent qualifiers in case of multiple entries warning about mx used while domain has null MX record warning for superfluous prefix lengths (/32 resp. /128) warning for zero prefix lengths (/0)","ref":"changelog.html#v0-3-0-2021-11-26","title":"Changelog - [v0.3.0] - 2021-11-26","type":"extras"},{"doc":"Changed verdict output includes owner domain and contact (also in csv-output) ipt logs show spf terms rather than their raw token logging to stderr now shows the domain in front, so redirecting stderr to a log file means the messages can be related to the domain being checked at that time. added warning when ?all or +all is used Fixed url for rfc7208 test suite use :dns (not :ipt) when logging dns additions to the cache","ref":"changelog.html#v0-2-0-2021-11-21","title":"Changelog - [v0.2.0] - 2021-11-21","type":"extras"},{"doc":"Fix url for License badge","ref":"changelog.html#v0-1-1-2021-11-20","title":"Changelog - [v0.1.1] - 2021-11-20","type":"extras"},{"doc":"Initial public version","ref":"changelog.html#v0-1-0-2021-11-20","title":"Changelog - [v0.1.0] - 2021-11-20","type":"extras"},{"doc":"Copyright (c) 2021 hertogp Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"spfcheck is a command line tool to examine and debug SPF records.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Usage: spfcheck [options] [sender ...] where sender = [localpart@]domain and localpart defaults to 'postmaster' Options: -H, --help print this message and exit -d, --dns=filepath file with DNS RR records to prepopulate the DNS cache -h, --helo=string sending MTA helo/ehlo identity (defaults to nil) -i, --ip=string sending MTA IPv4/IPv6 address (defaults to 127.0.0.1) -r, --report=string either &quot;all&quot; or one of more letters of &quot;vsewpdat&quot; (see below) -v, --verbosity set logging noise level (0..5), default is 4 (informational) -w, --width=NUM limits line length to increase readability (defaults to 60) --no-color turn off colors for log messages --no-markdown turn off markdown formatting for reports The default is to simply print the verdict and some stats to stdout and print notification messages to stderr. spfcheck passes the rfc7208 test suite and should be reasonably rfc7208 compliant. % spfcheck example.com --no-color example.com %spf[0]-spf-note: &gt; spfcheck(example.com, 127.0.0.1, example.com) example.com %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.com - [&quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot;, &quot;v=spf1 -all&quot;] example.com %spf[0]-eval-note: &gt; spf[0] -all - matches example.com %spf[0]-dns-info: &gt; DNS QUERY (2) soa example.com - [{&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600}] domain : example.com ip : 127.0.0.1 sender : example.com verdict : fail reason : spf[0] -all owner : example.com contact : noc@dns.icann.org num_spf : 1 num_dnsm : 0 num_dnsq : 1 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 0 explanation:","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"If no sender is given on the command line, spfcheck will read stdin for the domains (and options) to check. In this case, the verdict(s) are output on stdout in csv-format as each domain is (sequentially) evaluated. % cat senders.txt example.com me@example.net -i 1.2.3.4 % cat domains.txt | spfcheck -v 0 domain,ip,sender,verdict,reason,owner,contact,num_spf,num_dnsm,num_dnsq,num_dnsv,num_checks,num_warn,num_error,duration,explanation &quot;example.com&quot;,&quot;127.0.0.1&quot;,&quot;example.com&quot;,:fail,&quot;spf[0] -all&quot;,&quot;example.com&quot;,&quot;noc@dns.icann.org&quot;,1,0,1,0,1,0,0,1,&quot;&quot; &quot;example.net&quot;,&quot;1.2.3.4&quot;,&quot;me@example.net&quot;,:fail,&quot;spf[0] -all&quot;,&quot;example.net&quot;,&quot;noc@dns.icann.org&quot;,1,0,1,0,1,0,0,0,&quot;&quot;","ref":"readme.html#batchmode","title":"Overview - Batchmode","type":"extras"},{"doc":"The -d flag can be used to either point to local file with RR-records or specify DNS data on the command line. If the file exists, it is read and used to prepopulate the cache. Otherwise, the text will be read as DNS data. This makes it possible to try out records before publishing them in DNS. That file should contain 1 RR record per line using the simple domain type rdata format. All domain 's are taken to be relative to root ('.'). % spfcheck example.com -v 0 -d &quot;example.com TXT v=spf1 +all&quot; domain : example.com ip : 127.0.0.1 sender : example.com verdict : pass reason : spf[0] +all num_spf : 1 num_dnsm : 0 num_dnsq : 1 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 1 explanation: # Or using a file % cat tmp/zonedata.txt # comments are ignored as are empty lines example.com TXT v=spf1 -all exp=why.%{d} example.com TXT just another txt record why.example.com TXT %{d}: %{i} is not one of our MTAs % spfcheck example.com -v 0 -d tmp/zonedata.txt domain : example.com ip : 127.0.0.1 sender : example.com verdict : fail reason : spf[0] -all num_spf : 1 num_dnsm : 0 num_dnsq : 2 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 1 explanation: example.com: 127.0.0.1 is not one of our MTAs spfcheck counts the number of dns mechanisms seen (dnsm), the number of queries performed (dnsq) and the number of void dns queries seen (dnsv). If the evaluation took more than 10 dns mechanisms, the verdict is modified accordingly.","ref":"readme.html#dns-flag","title":"Overview - DNS flag","type":"extras"},{"doc":"The -h allows for setting the EHLO domain name and defaults to given sender . Note that spfcheck only checks SPF for sender , so this is only useful when checking the expansion of the %{h} -macro in a policy.","ref":"readme.html#helo-flag","title":"Overview - Helo flag","type":"extras"},{"doc":"The -i flag is used to set sender's IP to either an IPv4 or an IPv6 address, it defaults to 127.0.0.1 as an unlikely address to be authorized by anyone. The goal is to go down the rabbit hole as far as possible and check the entire nested SPF policy for given sender .","ref":"readme.html#ip-flag","title":"Overview - Ip flag","type":"extras"},{"doc":"The --no-color flag disables the use of colors in log messages, which is better when redirecting logging to a file. For example: %cat tmp/domains.txt example.com me@example.net -i 1.2.3.4 % cat tmp/domains.txt | spfcheck -v 5 --no-color 2&gt;tmp/log.txt &gt; tmp/checked.csv % cat tmp/log.txt example.com %spf[0]-ctx-debug: &gt; created context for example.com example.com %spf[0]-spf-note: &gt; spfcheck(example.com, 127.0.0.1, example.com) example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot; example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;v=spf1 -all&quot; example.com %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.com - [&quot;v=spf1 -all&quot;, &quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot;] example.com %spf[0]-eval-info: &gt; spf[0] -all - matches example.com %spf[0]-dns-debug: &gt; added {example.com, soa} -&gt; {&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600} example.com %spf[0]-dns-info: &gt; DNS QUERY (2) soa example.com - [{&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600}] example.net %spf[0]-ctx-debug: &gt; created context for example.net example.net %spf[0]-spf-note: &gt; spfcheck(example.net, 1.2.3.4, me@example.net) example.net %spf[0]-dns-debug: &gt; added {example.net, txt} -&gt; &quot;v=spf1 -all&quot; example.net %spf[0]-dns-debug: &gt; added {example.net, txt} -&gt; &quot;5fpl1ghm7scnth0907z0pft8c79lvc8t&quot; example.net %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.net - [&quot;5fpl1ghm7scnth0907z0pft8c79lvc8t&quot;, &quot;v=spf1 -all&quot;] example.net %spf[0]-eval-info: &gt; spf[0] -all - matches example.net %spf[0]-dns-debug: &gt; added {example.net, soa} -&gt; {&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600} example.net %spf[0]-dns-info: &gt; DNS QUERY (2) soa example.net - [{&quot;ns.icann.org&quot;, &quot;noc.dns.icann.org&quot;, 2021111701, 7200, 3600, 1209600, 3600}]","ref":"readme.html#no-color","title":"Overview - No color","type":"extras"},{"doc":"The -r flag can be used to print out some information, topics include: v the verdict and some statistics s the spf records seen and their authority information e the errors seen w the warnings seen p the prefixes collected d DNS information collected a the AST for the (first) SPF record t the tokens for the (last) SPF record seen In case no -r flag is used, spfcheck will simply print out the verdict. % spfcheck example.com -v 0 -r s --no-markdown [0] example.com -- (example.com, noc@dns.icann.org) v=spf1 -all Here, the SPF section lists all SPF records seen along with the authoritative domain and its DNS admin email.","ref":"readme.html#report-flag","title":"Overview - Report flag","type":"extras"},{"doc":"The -v flag controls the verbosity level of logging on stderr: 0 - no messages at all 1 - errors 2 - warnings 3 - notifications 4 - informational 5 - debug % spfcheck example.com -v 5 --no-color example.com %spf[0]-ctx-debug: &gt; created context for example.com example.com %spf[0]-spf-note: &gt; spfcheck(example.com, 127.0.0.1, example.com) example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;v=spf1 -all&quot; example.com %spf[0]-dns-debug: &gt; added {example.com, txt} -&gt; &quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot; example.com %spf[0]-dns-info: &gt; DNS QUERY (1) txt example.com - [&quot;8j5nfqld20zpcyr8xjw0ydcfq9rk8hgm&quot;, &quot;v=spf1 -all&quot;] example.com %spf[0]-spf-note: &gt; SPF (0): [&quot;v=spf1 -all&quot;] example.com %spf[0]-eval-info: &gt; spf[0] -all - matches domain : example.com ip : 127.0.0.1 sender : example.com verdict : fail reason : spf[0] -all num_spf : 1 num_dnsm : 0 num_dnsq : 1 num_dnsv : 0 num_checks : 1 num_warn : 0 num_error : 0 duration : 1 explanation:","ref":"readme.html#verbosity-flag","title":"Overview - Verbosity flag","type":"extras"},{"doc":"Finally, the -w flag can be used to control the width used when printing information of certain topics. Primarily meant so a markdown formatted report can be easily converted to pdf. Default value is 60, but you can make it as wide as necessary.","ref":"readme.html#width-flag","title":"Overview - Width flag","type":"extras"},{"doc":"You can install spfcheck as an escript: mix escript.install hex spfcheck After installation, ~/.mix/escripts/spfcheck invokes the escript. Or use it in a project by adding spfcheck to the list of dependencies in mix.exs : def deps do [ { :spfcheck , &quot;~&gt; 0.3.0&quot; } ] end","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"}]